/*-------------------------------------------------*/
/* In-circuit checker   (C)ChaN, 2008              */
/* modified for tiny10                             */
/*-------------------------------------------------*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#define	SYSCLK		128000		/* CKSEL[1:0]=11, CKDIV8=1 */


EMPTY_INTERRUPT(PCINT0_vect);


void delay_ms (uint16_t dly)
{
  uint8_t n, d;

  do {
    n = 31;
    do d = PINB; while (--n);
  } while (--dly);
}


void beep (uint8_t t)
{
  if (t) {
    OCR0AL = t;
    OCR0BL = t / 2;
    TCCR0A = _BV(COM0B1)|_BV(WGM00);
    t = _BV(WGM02)|0b001;
  }
  TCCR0B = t;
}

uint16_t sdt;

int main (void)
{
  uint8_t ad;

  static const int8_t tone[] = {
    SYSCLK / 4000,	/* 0..5 ohm */
    SYSCLK / 2640,	/* 5..15 ohm */
    SYSCLK / 1742,	/* 15..25 ohm */
    SYSCLK / 1150,	/* 25..35 ohm */
    SYSCLK / 760	/* 35..45 ohm */
  };

  CCP = 0xD8;
  CLKMSR = 0x01;  // internal 128kHz
  CCP = 0xD8;
  CLKPSR = 0x00; // prescale = 1

  PORTB = 0b0001;	/* Initialize I/O port */
  PUEB  = 0b0100;	/* Pull-Up enable on BP0 */
  DDRB  = 0b0011;	/* BP0=LED, PB1=BZ, PB2=SW, PB3=ADC & Bias(Internap P-up) */

  ACSR = _BV(ACD);	/* Disable analog comp. */

  PCICR = _BV(PCIE0);
  PCMSK = _BV(PCINT2);	/* Enable PCINT2 */
  // EICRA = 0x01;		/* logical change on INT0 gegerate interrupt  */

  sei();
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);

  for (;;) {
    PORTB = 0b0001;			/* LED OFF*/ 
    PUEB  = 0b0100;			/* Disable bias circuit */
    ADCSRA = 0;				/* Stop ADC */
    while (bit_is_clear(PINB,PB2))	/* Wait for button released */
      delay_ms(20);
    sleep_mode();			/* Enter Power-Down mode */
    delay_ms(50);			/* After leaving Power-Down mode, delay 50ms */
    if (bit_is_set(PINB,PB2))		/* When it is accidental wakeup, re-enter Power-Down mode */
      continue;
    PORTB = 0b0000;			/* LED ON */
    PUEB  = 0b1100;			/* Enable bias circuit */
    ADMUX = 0b11;			/* ADC Ch=3, Vref=Vcc */
    sdt = 0;				/* Initialize shutdown timer */
    while (bit_is_clear(PINB,PB2))	/* Wait for button released */
      delay_ms(20);

    beep(tone[0]);			/* Startup beep */
    delay_ms(100);
    beep(0);

    do {
      ADCSRA = _BV(ADEN)|_BV(ADSC);	/* Start conversion */
      delay_ms(10);			/* 10ms */
      ad = ADCL;			/* Get input voltage */
      if (ad < 2) {
	beep(tone[ad]);			/* Start beep at frequency corresponds to the resistance */
	sdt = 0;			/* Clear shutdown timer */
      } else {
	beep(0);			/* Sto beep */
      }
    } while (bit_is_set(PINB,PB2) && (++sdt < 10000));

    if (bit_is_set(PINB,PB2)) {		/* Beep on auto power off */
      beep(tone[0]);
      delay_ms(500);
    }
    beep(0);
  }
}

/* EOF */
